﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Airbus
{
    //класс, отвечающий за прорисовку и перемещение объекта
    public class DrawningAirbus
    {
        public EntityAirbus Airbus { get; protected set; } //класс-сущность

        protected float _startPosX; //левая координата отрисовки

        protected float _startPosY; //верхняя координата отрисовки

        private int? _pictureWidth = null; //ширина окна отрисовки
        
        private int? _pictureHeight = null; //высота окна отрисовки

        protected readonly int _airbusWidth = 50; //ширина отрисовки самолёта

        protected readonly int _airbusHeight = 16; //высота отрисовки самолёта

        //инициализаци свойств
        public DrawningAirbus(int speed, float weight, Color corpusColor)
        {
            Airbus = new EntityAirbus(speed, weight, corpusColor);
        }

        protected DrawningAirbus(int speed, float weight, Color corpusColor, int airbusWidth, int airbusHeight) :
            this(speed, weight, corpusColor)   
        {
            _airbusWidth = airbusWidth;
            _airbusHeight = airbusHeight;
        }

        //установка координат позиции самолёта
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x < 0 || y < 0 || width < x + _airbusWidth || height < y + _airbusHeight)
            {
                _pictureHeight = null;
                _pictureWidth = null;

                return;
            }

            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }

        //изменение направления движения
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }

            switch (direction)
            {
                case Direction.Right: //вправо
                    if (_startPosX + _airbusWidth + Airbus.Step < _pictureWidth)
                    {
                        _startPosX += Airbus.Step;
                    }
                    break;

                case Direction.Left: //влево
                    if (_startPosX - _airbusWidth - Airbus.Step > 0)
                    {
                        _startPosX -= Airbus.Step;
                    }
                    break;

                case Direction.Up: //вверх
                    if (_startPosY - _airbusHeight - Airbus.Step > 0)
                    {
                        _startPosY -= Airbus.Step;
                    }
                    break;

                case Direction.Down: //вниз
                    if (_startPosY + _airbusHeight + Airbus.Step < _pictureHeight)
                    {
                        _startPosY += Airbus.Step;
                    }
                    break;
            }
        }

        //отрисовка самолёта
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0 || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush selectedBrush = new SolidBrush(Airbus.CorpusColor);
            Brush darcBlue = new SolidBrush(Color.DarkSlateBlue);
            Brush darkBrush = new SolidBrush(Color.Black);

            //корпус
            g.FillRectangle(selectedBrush, _startPosX, _startPosY + 10, 40, 10);
            g.DrawRectangle(pen, _startPosX, _startPosY + 10, 40, 10);

            //крыло
            g.FillRectangle(selectedBrush, _startPosX + 10, _startPosY + 13, 20, 2);
            g.FillRectangle(selectedBrush, _startPosX + 12, _startPosY + 15, 16, 2);
            g.DrawRectangle(pen, _startPosX + 10, _startPosY + 13, 20, 2);
            g.DrawRectangle(pen, _startPosX + 12, _startPosY + 15, 16, 2);

            //хвостовое крыло
            Point[] point_tailWing = new Point[]
            {
                new Point((int)_startPosX, (int)_startPosY + 10), 
                new Point((int)_startPosX, (int)_startPosY),
                new Point((int)_startPosX + 10, (int)_startPosY + 10), 
                new Point((int)_startPosX, (int)_startPosY + 10)
            };

            g.FillPolygon(selectedBrush, point_tailWing);
            g.DrawLine(pen, _startPosX, _startPosY + 10, _startPosX, _startPosY);
            g.DrawLine(pen, _startPosX, _startPosY, _startPosX + 10, _startPosY + 10);

            //заднее поперечное крыло
            g.FillEllipse(selectedBrush, _startPosX - 3, _startPosY + 7, 10, 5);
            g.DrawEllipse(pen, _startPosX - 3, _startPosY + 7, 10, 5);

            //нос самолёта
            //верхняя половина
            Point[] point_upperBow = new Point[]
            {
                new Point((int)_startPosX + 40, (int)_startPosY + 15),
                new Point((int)_startPosX + 50, (int)_startPosY + 16),
                new Point((int)_startPosX + 40, (int)_startPosY + 10),
                new Point((int)_startPosX + 40, (int)_startPosY + 15)
            };

            //нижняя половина
            Point[] point_lowerBow = new Point[]
            {
                new Point((int)_startPosX + 40, (int)_startPosY + 15),
                new Point((int)_startPosX + 40, (int)_startPosY + 20),
                new Point((int)_startPosX + 50, (int)_startPosY + 16),
                new Point((int)_startPosX + 40, (int)_startPosY + 15),
            };

            g.FillPolygon(darcBlue, point_upperBow);
            g.FillPolygon(selectedBrush, point_lowerBow);
            g.DrawLine(pen, _startPosX + 40, _startPosY + 15, _startPosX + 50, _startPosY + 16);
            g.DrawLine(pen, _startPosX + 40, _startPosY + 10, _startPosX + 47, _startPosY + 15);
            g.DrawLine(pen, _startPosX + 40, _startPosY + 20, _startPosX + 50, _startPosY + 15);

            //задние шасси
            g.FillRectangle(darkBrush, _startPosX + 10, _startPosY + 21, 2, 2);
            g.FillRectangle(darkBrush, _startPosX + 12, _startPosY + 23, 4, 4);
            g.FillRectangle(darkBrush, _startPosX + 8, _startPosY + 23, 2, 4);
            g.DrawRectangle(pen, _startPosX + 10, _startPosY + 21, 2, 2);
            g.DrawRectangle(pen, _startPosX + 12, _startPosY + 23, 4, 4);
            g.DrawRectangle(pen, _startPosX + 8, _startPosY + 23, 2, 4);

            //переднее шасси
            g.FillRectangle(darkBrush, _startPosX + 36, _startPosY + 21, 2, 2);
            g.FillRectangle(darkBrush, _startPosX + 36, _startPosY + 23, 4, 4);
            g.DrawRectangle(pen, _startPosX + 35, _startPosY + 21, 2, 2);
            g.DrawRectangle(pen, _startPosX + 35, _startPosY + 23, 4, 4);
            
        }

        //смена границ формы отрисовки
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;

            if (_pictureWidth <= _airbusWidth || _pictureHeight <= _airbusHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }

            if (_startPosX + _airbusWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _airbusWidth;
            }

            if (_startPosY + _airbusHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _airbusHeight;
            }
        }

        //получение текущей позиции объекта
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosY, _startPosX + _airbusWidth, _startPosY + _airbusHeight);
        }
    }
}
